@@ -1,24 +1,9 @@
 /*
+ * Embedded Linux library
+ * Copyright (C) 2011-2014  Intel Corporation
+ * Copyright (C) 2017  Codecoup
  *
- *  Embedded Linux library
- *
- *  Copyright (C) 2011-2014  Intel Corporation. All rights reserved.
- *  Copyright (C) 2017  Codecoup. All rights reserved.
- *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation; either
- *  version 2.1 of the License, or (at your option) any later version.
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
- *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #ifdef HAVE_CONFIG_H
@@ -28,6 +13,7 @@
 #include "dbus.h"
 #include "dbus-client.h"
 #include "queue.h"
+#include "useful.h"
 #include "private.h"
 
 struct l_dbus_client {
@@ -301,6 +287,9 @@ LIB_EXPORT uint32_t l_dbus_proxy_method_call(struct l_dbus_proxy *proxy,
 {
 	struct method_call_request *req;
 
+	if (unlikely(!proxy))
+		return 0;
+
 	req = l_new(struct method_call_request, 1);
 	req->proxy = proxy;
 	req->setup = setup;
@@ -411,7 +400,7 @@ static struct l_dbus_proxy *dbus_proxy_new(struct l_dbus_client *client,
 	proxy->interface = l_strdup(interface);
 	proxy->path = l_strdup(path);
 	proxy->properties = l_queue_new();
-	proxy->pending_calls = l_queue_new();;
+	proxy->pending_calls = l_queue_new();
 
 	l_queue_push_tail(client->proxies, proxy);
 
@@ -546,7 +535,8 @@ static void get_managed_objects_reply(struct l_dbus_message *message,
 	if (l_dbus_message_is_error(message))
 		return;
 
-	l_dbus_message_get_arguments(message, "a{oa{sa{sv}}}", &objects);
+	if (!l_dbus_message_get_arguments(message, "a{oa{sa{sv}}}", &objects))
+		return;
 
 	while (l_dbus_message_iter_next_entry(&objects, &path, &object))
 		parse_object(client, path, &object);
@@ -647,8 +637,8 @@ LIB_EXPORT void l_dbus_client_destroy(struct l_dbus_client *client)
 		client->proxy_cb_data_destroy(client->proxy_cb_data);
 
 	if (client->objects_call)
-		l_dbus_cancel(client->dbus, client->objects_call)
-;
+		l_dbus_cancel(client->dbus, client->objects_call);
+
 	l_queue_destroy(client->proxies,
 				(l_queue_destroy_func_t)dbus_proxy_destroy);
 
